<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model\Repository\Product;

use B2W\SkyHub\Contract\Entity\Product\VariationEntityInterface;
use B2W\SkyHub\Contract\Repository\Product\VariationRepositoryInterface;
use B2W\SkyHub\Model\Resource\Collection;
use B2W\SkyHub\Model\Resource\Select;
use B2W\SkyHub\Model\Transformer\Product\Variation\DbToEntity;

class VariationDbRepository implements VariationRepositoryInterface
{
    /**
     * @param \B2W\SkyHub\Contract\Entity\ProductEntityInterface|\WP_Post $product
     * @return Collection
     * @throws \Exception
     */
    public function load($product)
    {
        $filter = array(
            'post_status'   => array('publish'),
            'numberposts'   => null,
            'post_type'     => VariationEntityInterface::POST_TYPE,
            'post_parent'   => $product instanceof \B2W\SkyHub\Contract\Entity\ProductEntityInterface
                ? $product->getId()
                : $product->ID
        );

        $posts      = get_posts($filter);
        $collection = new Collection();

        foreach ($posts as $post) {
            $variation = $this->one($post);
            $collection->addItem($variation);
        }

        return $collection;
    }

    /**
     * @param $id
     * @return mixed|null
     * @throws \Exception
     */
    public function one($id)
    {
        if ($id instanceof \WP_Post) {
            $post = $id;
        } else {
            $post = get_post($id);
        }

        /** @var \B2W\SkyHub\Model\Transformer\Product\Variation\DbToEntity $transformer */
        $transformer = new DbToEntity();
        $transformer->setPost($post);
        $transformer->setMeta(get_post_meta($post->ID));
        return $transformer->convert();
    }

    /**
     * @param $sku
     * @return bool|mixed|null
     * @throws \Exception
     */
    public function sku($sku)
    {
        global $wpdb;

        $sku = sanitize_text_field($sku);

        $select = new Select();
        $select->column('post_id');
        $select->from('postmeta');
        $select->where("meta_key = '_sku'");
        $select->where("meta_value= '$sku'");

        foreach ($wpdb->get_results($select) as $row) {
            return $this->one($row->post_id);
        }

        return false;
    }
}
