<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\View\Admin\Form;


use B2W\SkyHub\View\Admin\Form\Field\Select;
use B2W\SkyHub\View\Admin\Form\Field\Text;
use B2W\SkyHub\View\Template;

/**
 * Class Fieldset
 * @package B2W\SkyHub\View\Admin\Form
 */
class Fieldset extends Template
{
    protected $_template = 'admin/form/fieldset.php';
    /**
     * @var null
     */
    protected $_name   = null;
    /**
     * @var null
     */
    protected $_title  = null;
    /**
     * @var array
     */
    protected $_fields = array();

    /**
     * @return null
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @param null $name
     */
    public function setName($name)
    {
        $this->_name = $name;
    }

    /**
     * @return null
     */
    public function getTitle()
    {
        return $this->_title;
    }

    /**
     * @param null $title
     */
    public function setTitle($title)
    {
        $this->_title = $title;
    }

    /**
     * @param $type
     * @param $name
     * @param $attributes
     * @return bool|mixed
     */
    public function addField($type, $name, $attributes = array())
    {
        if (isset($this->_fields[$name])) {
            return false;
        }

        $label      = isset($attributes['label']) ? $attributes['label'] : '';
        $note       = isset($attributes['note'])
            ? (is_array($attributes['note']) ? $attributes['note'] : array($attributes['note']))
            : array();
        $options    = isset($attributes['options']) ? $attributes['options'] : array();
        $value      = isset($attributes['value']) ? $attributes['value'] : '';
        $id         = isset($attributes['id']) ? $attributes['id'] : $name;

        switch ($type) {
            case 'text' :
                $this->addTextField($name, $label, $id, $value, $note);
                break;
            case 'select' :
                $this->addSelectField($name, $label, $id, $value, $options, $note);
                break;
        }

        return $this->_fields[$name];
    }

    /**
     * @return array
     */
    public function getFields()
    {
        return $this->_fields;
    }

    public function addTextField($name, $label = '', $id = '', $value = '', $note = array())
    {
        $field = new Text();
        $field->setName($name);
        $field->setLabel($label);
        $field->setId($id);
        $field->setValue($value);

        foreach ($note as $v) {
            $field->addNote($v);
        }

        $this->_fields[$name] = $field;

        return $field;
    }

    public function addSelectField($name, $label = '', $id = '', $value = '', $options = array(), $note = array())
    {
        $field = new Select();
        $field->setName($name);
        $field->setLabel($label);
        $field->setId($id);
        $field->setValue($value);
        $field->setOptions($options);

        foreach ($note as $v) {
            $field->addNote($v);
        }

        $this->_fields[$name] = $field;

        return $field;
    }
}
